@@ -182,8 +182,7 @@ class PauseAtHeight(Script):
                         "Repetier": "Repetier"
                     },
                     "default_value": "RepRap (Marlin/Sprinter)",
-                    "enabled": false,
-                    "default_value": ""
+                    "enabled": false
                 },
                 "custom_gcode_before_pause":
                 {
@@ -339,11 +338,6 @@ class PauseAtHeight(Script):
                     if current_layer < pause_layer - nbr_negative_layers:
                         continue
 
-                # Get X and Y from the next layer (better position for
-                # the nozzle)
-                next_layer = data[index + 1]
-                x, y = self.getNextXY(next_layer)
-
                 prev_layer = data[index - 1]
                 prev_lines = prev_layer.split("\n")
                 current_e = 0.
@@ -354,6 +348,13 @@ class PauseAtHeight(Script):
                     current_e = self.getValue(prevLine, "E", -1)
                     if current_e >= 0:
                         break
+                # and also find last X,Y
+                for prevLine in reversed(prev_lines):
+                    if prevLine.startswith(("G0", "G1", "G2", "G3")):
+                        if self.getValue(prevLine, "X") is not None and self.getValue(prevLine, "Y") is not None:
+                            x = self.getValue(prevLine, "X")
+                            y = self.getValue(prevLine, "Y")
+                            break
 
                 # Maybe redo the last layer.
                 if redo_layer:
@@ -455,7 +456,7 @@ class PauseAtHeight(Script):
                         prepend_gcode += self.putValue(G = 1, E = -retraction_amount, F = 6000) + "\n"
 
                     #Move the head back
-                    prepend_gcode += self.putValue(G = 1, Z = current_z + 1, F = 300) + "\n"
+                    prepend_gcode += self.putValue(G = 1, Z = current_z, F = 300) + "\n"
                     prepend_gcode += self.putValue(G = 1, X = x, Y = y, F = 9000) + "\n"
                     if retraction_amount != 0:
                         prepend_gcode += self.putValue(G = 1, E = retraction_amount, F = 6000) + "\n"
