@@ -1,4 +1,4 @@
-# Copyright (c) 2019 Ultimaker B.V.
+# Copyright (c) 2020 Ultimaker B.V.
 # Toolbox is released under the terms of the LGPLv3 or higher.
 
 import json
@@ -9,22 +9,20 @@ from typing import cast, Any, Dict, List, Set, TYPE_CHECKING, Tuple, Optional, U
 from PyQt5.QtCore import QObject, pyqtProperty, pyqtSignal, pyqtSlot
 from PyQt5.QtNetwork import QNetworkAccessManager, QNetworkRequest, QNetworkReply
 
+from UM.Extension import Extension
 from UM.Logger import Logger
 from UM.PluginRegistry import PluginRegistry
-from UM.Extension import Extension
-from UM.i18n import i18nCatalog
+from UM.TaskManagement.HttpRequestScope import JsonDecoratorScope
 from UM.Version import Version
-
+from UM.i18n import i18nCatalog
 from cura import ApplicationMetadata
-
 from cura.CuraApplication import CuraApplication
 from cura.Machines.ContainerTree import ContainerTree
-
-from .CloudApiModel import CloudApiModel
+from cura.UltimakerCloud.UltimakerCloudScope import UltimakerCloudScope
 from .AuthorsModel import AuthorsModel
+from .CloudApiModel import CloudApiModel
 from .CloudSync.LicenseModel import LicenseModel
 from .PackagesModel import PackagesModel
-from .UltimakerCloudScope import UltimakerCloudScope
 
 if TYPE_CHECKING:
     from UM.TaskManagement.HttpRequestData import HttpRequestData
@@ -39,22 +37,21 @@ try:
 except ImportError:
     CuraMarketplaceRoot = DEFAULT_MARKETPLACE_ROOT
 
-# todo Remove license and download dialog, use SyncOrchestrator instead
 
-##  Provides a marketplace for users to download plugins an materials
 class Toolbox(QObject, Extension):
+    """Provides a marketplace for users to download plugins an materials"""
+
     def __init__(self, application: CuraApplication) -> None:
         super().__init__()
 
         self._application = application  # type: CuraApplication
 
-        self._sdk_version = ApplicationMetadata.CuraSDKVersion  # type: Union[str, int]
-
         # Network:
         self._download_request_data = None  # type: Optional[HttpRequestData]
         self._download_progress = 0  # type: float
         self._is_downloading = False  # type: bool
-        self._scope = UltimakerCloudScope(application)  # type: UltimakerCloudScope
+        self._cloud_scope = UltimakerCloudScope(application)  # type: UltimakerCloudScope
+        self._json_scope = JsonDecoratorScope(self._cloud_scope)  # type: JsonDecoratorScope
 
         self._request_urls = {}  # type: Dict[str, str]
         self._to_update = []  # type: List[str] # Package_ids that are waiting to be updated
@@ -78,10 +75,15 @@ class Toolbox(QObject, Extension):
         self._plugins_showcase_model = PackagesModel(self)
         self._plugins_available_model = PackagesModel(self)
         self._plugins_installed_model = PackagesModel(self)
-
+        self._plugins_installed_model.setFilter({"is_bundled": "False"})
+        self._plugins_bundled_model = PackagesModel(self)
+        self._plugins_bundled_model.setFilter({"is_bundled": "True"})
         self._materials_showcase_model = AuthorsModel(self)
         self._materials_available_model = AuthorsModel(self)
         self._materials_installed_model = PackagesModel(self)
+        self._materials_installed_model.setFilter({"is_bundled": "False"})
+        self._materials_bundled_model = PackagesModel(self)
+        self._materials_bundled_model.setFilter({"is_bundled": "True"})
         self._materials_generic_model = PackagesModel(self)
 
         self._license_model = LicenseModel()
@@ -120,7 +122,7 @@ class Toolbox(QObject, Extension):
     onIsDownloadingChanged = pyqtSignal()
     restartRequiredChanged = pyqtSignal()
     installChanged = pyqtSignal()
-    enabledChanged = pyqtSignal()
+    toolboxEnabledChanged = pyqtSignal()
 
     # UI changes
     viewChanged = pyqtSignal()
@@ -131,8 +133,9 @@ class Toolbox(QObject, Extension):
     closeLicenseDialog = pyqtSignal()
     uninstallVariablesChanged = pyqtSignal()
 
-    ##  Go back to the start state (welcome screen or loading if no login required)
     def _restart(self):
+        """Go back to the start state (welcome screen or loading if no login required)"""
+
         # For an Essentials build, login is mandatory
         if not self._application.getCuraAPI().account.isLoggedIn and ApplicationMetadata.IsEnterpriseVersion:
             self.setViewPage("welcome")
@@ -146,13 +149,6 @@ class Toolbox(QObject, Extension):
         self._package_used_materials = []  # type: List[Tuple[GlobalStack, str, str]]
         self._package_used_qualities = []  # type: List[Tuple[GlobalStack, str, str]]
 
-    @pyqtSlot(str, int)
-    def ratePackage(self, package_id: str, rating: int) -> None:
-        url = "{base_url}/packages/{package_id}/ratings".format(base_url = CloudApiModel.api_url, package_id = package_id)
-        data = "{\"data\": {\"cura_version\": \"%s\", \"rating\": %i}}" % (Version(self._application.getVersion()), rating)
-
-        self._application.getHttpRequestManager().put(url, data = data.encode(), scope = self._scope)
-
     def getLicenseDialogPluginFileLocation(self) -> str:
         return self._license_dialog_plugin_file_location
 
@@ -212,7 +208,7 @@ class Toolbox(QObject, Extension):
 
         self._dialog.show()
         # Apply enabled/disabled state to installed plugins
-        self.enabledChanged.emit()
+        self.toolboxEnabledChanged.emit()
 
     def _createDialog(self, qml_name: str) -> Optional[QObject]:
         Logger.log("d", "Marketplace: Creating dialog [%s].", qml_name)
@@ -227,7 +223,7 @@ class Toolbox(QObject, Extension):
             "licenseModel": self._license_model
         })
         if not dialog:
-            raise Exception("Failed to create Marketplace dialog")
+            return None
         return dialog
 
     def _convertPluginMetadata(self, plugin_data: Dict[str, Any]) -> Optional[Dict[str, Any]]:
@@ -290,9 +286,11 @@ class Toolbox(QObject, Extension):
             self._old_plugin_metadata = {k: v for k, v in self._old_plugin_metadata.items() if k in self._old_plugin_ids}
 
             self._plugins_installed_model.setMetadata(all_packages["plugin"] + list(self._old_plugin_metadata.values()))
+            self._plugins_bundled_model.setMetadata(all_packages["plugin"] + list(self._old_plugin_metadata.values()))
             self.metadataChanged.emit()
         if "material" in all_packages:
             self._materials_installed_model.setMetadata(all_packages["material"])
+            self._materials_bundled_model.setMetadata(all_packages["material"])
             self.metadataChanged.emit()
 
     @pyqtSlot(str)
@@ -305,10 +303,13 @@ class Toolbox(QObject, Extension):
         self.restartRequiredChanged.emit()
         return package_id
 
-    ##  Check package usage and uninstall
-    #   If the package is in use, you'll get a confirmation dialog to set everything to default
     @pyqtSlot(str)
     def checkPackageUsageAndUninstall(self, package_id: str) -> None:
+        """Check package usage and uninstall
+
+        If the package is in use, you'll get a confirmation dialog to set everything to default
+        """
+
         package_used_materials, package_used_qualities = self._package_manager.getMachinesUsingPackage(package_id)
         if package_used_materials or package_used_qualities:
             # Set up "uninstall variables" for resetMaterialsQualitiesAndUninstall
@@ -346,10 +347,13 @@ class Toolbox(QObject, Extension):
         if self._confirm_reset_dialog is not None:
             self._confirm_reset_dialog.close()
 
-    ##  Uses "uninstall variables" to reset qualities and materials, then uninstall
-    #   It's used as an action on Confirm reset on Uninstall
     @pyqtSlot()
     def resetMaterialsQualitiesAndUninstall(self) -> None:
+        """Uses "uninstall variables" to reset qualities and materials, then uninstall
+
+        It's used as an action on Confirm reset on Uninstall
+        """
+
         application = CuraApplication.getInstance()
         machine_manager = application.getMachineManager()
         container_tree = ContainerTree.getInstance()
@@ -412,8 +416,9 @@ class Toolbox(QObject, Extension):
         self._restart_required = True
         self.restartRequiredChanged.emit()
 
-    ##  Actual update packages that are in self._to_update
     def _update(self) -> None:
+        """Actual update packages that are in self._to_update"""
+
         if self._to_update:
             plugin_id = self._to_update.pop(0)
             remote_package = self.getRemotePackage(plugin_id)
@@ -427,16 +432,17 @@ class Toolbox(QObject, Extension):
         if self._to_update:
             self._application.callLater(self._update)
 
-    ##  Update a plugin by plugin_id
     @pyqtSlot(str)
     def update(self, plugin_id: str) -> None:
+        """Update a plugin by plugin_id"""
+
         self._to_update.append(plugin_id)
         self._application.callLater(self._update)
 
     @pyqtSlot(str)
     def enable(self, plugin_id: str) -> None:
         self._plugin_registry.enablePlugin(plugin_id)
-        self.enabledChanged.emit()
+        self.toolboxEnabledChanged.emit()
         Logger.log("i", "%s was set as 'active'.", plugin_id)
         self._restart_required = True
         self.restartRequiredChanged.emit()
@@ -444,7 +450,7 @@ class Toolbox(QObject, Extension):
     @pyqtSlot(str)
     def disable(self, plugin_id: str) -> None:
         self._plugin_registry.disablePlugin(plugin_id)
-        self.enabledChanged.emit()
+        self.toolboxEnabledChanged.emit()
         Logger.log("i", "%s was set as 'deactive'.", plugin_id)
         self._restart_required = True
         self.restartRequiredChanged.emit()
@@ -541,7 +547,7 @@ class Toolbox(QObject, Extension):
         self._application.getHttpRequestManager().get(url,
                                                       callback = callback,
                                                       error_callback = error_callback,
-                                                      scope=self._scope)
+                                                      scope=self._json_scope)
 
     @pyqtSlot(str)
     def startDownload(self, url: str) -> None:
@@ -554,7 +560,7 @@ class Toolbox(QObject, Extension):
                                                                      callback = callback,
                                                                      error_callback = error_callback,
                                                                      download_progress_callback = download_progress_callback,
-                                                                     scope=self._scope
+                                                                     scope=self._cloud_scope
                                                                      )
 
         self._download_request_data = request_data
@@ -602,7 +608,7 @@ class Toolbox(QObject, Extension):
         # Check for errors:
         if "errors" in json_data:
             for error in json_data["errors"]:
-                Logger.log("e", "Request type [%s] got response showing error: %s", error["title"])
+                Logger.log("e", "Request type [%s] got response showing error: %s", error.get("title", "No error title found"))
             self.setViewPage("errored")
             return
 
@@ -708,9 +714,10 @@ class Toolbox(QObject, Extension):
             self._active_package = package
             self.activePackageChanged.emit()
 
-    ##  The active package is the package that is currently being downloaded
     @pyqtProperty(QObject, fset = setActivePackage, notify = activePackageChanged)
     def activePackage(self) -> Optional[QObject]:
+        """The active package is the package that is currently being downloaded"""
+
         return self._active_package
 
     def setViewCategory(self, category: str = "plugin") -> None:
@@ -718,7 +725,7 @@ class Toolbox(QObject, Extension):
             self._view_category = category
             self.viewChanged.emit()
 
-    ## Function explicitly defined so that it can be called through the callExtensionsMethod
+    # Function explicitly defined so that it can be called through the callExtensionsMethod
     # which cannot receive arguments.
     def setViewCategoryToMaterials(self) -> None:
         self.setViewCategory("material")
@@ -758,6 +765,10 @@ class Toolbox(QObject, Extension):
     def pluginsInstalledModel(self) -> PackagesModel:
         return self._plugins_installed_model
 
+    @pyqtProperty(QObject, constant = True)
+    def pluginsBundledModel(self) -> PackagesModel:
+        return self._plugins_bundled_model
+
     @pyqtProperty(QObject, constant = True)
     def materialsShowcaseModel(self) -> AuthorsModel:
         return self._materials_showcase_model
@@ -770,6 +781,10 @@ class Toolbox(QObject, Extension):
     def materialsInstalledModel(self) -> PackagesModel:
         return self._materials_installed_model
 
+    @pyqtProperty(QObject, constant = True)
+    def materialsBundledModel(self) -> PackagesModel:
+        return self._materials_bundled_model
+
     @pyqtProperty(QObject, constant = True)
     def materialsGenericModel(self) -> PackagesModel:
         return self._materials_generic_model
